<?php
namespace App\Controller;

use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;
use \App\Util\Util;

class Dashboard {

    private $dashboardService;
    private $util;

    public function __construct(\App\Service\Dashboard $dashboardService, Util $util) 
    {
        $this->dashboardService = $dashboardService;
        $this->util = $util;
    }

    public function dashboard(Request $request, Response $response) 
    {
        try {
            $forceEmptyDashboard = $this->util->getEnv("FORCE_EMPTY_DASHBOARD") === 'YES';
            
            if ($forceEmptyDashboard) {
                $response->getBody()->write(json_encode([
                    'error'   => false,
                    'message' => 'FORCE_EMPTY_DASHBOARD',
                    'data'    => []
                ]));
                return $response->withHeader('Content-Type', 'application/json');
            }
            
            $params = $request->getParsedBody();

            $dtInicial = !empty($params['dtInicial']) ? $params['dtInicial'] : null;
            $dtFinal = !empty($params['dtFinal']) ? $params['dtFinal'] : null;

            if (empty($params['dtInicial']) || empty($params['dtFinal'])) {
                throw new \Exception("Data inicial/final não informada.");
            }
            
            $result = $this->dashboardService->dashboard($dtInicial, $dtFinal);

            $response->getBody()->write(json_encode([
                'error' => false,
                'message' => "Dados do Dashboard adquiridos com sucesso.",
                'data' => $result
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        } catch (\Exception $e) {
            $response->getBody()->write(json_encode([
                'error'   => true,
                'message' => $e->getMessage(),
                'data'    => []
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        }
    }
}