<?php

namespace App\Service;

class Address {

    private $addressFactory;
    private $util;

    public function __construct(
								\App\Factory\Address $addressFactory,
								\App\Util\Util $util
	) {
        $this->addressFactory = $addressFactory;
        $this->util = $util;
    }

    public function addAdditionalAddress($params) {
			$addressCode   = $this->util->newCode('ENDERECOCONS', 10);
			$streetAddress = $params['DSENDECONS'];
			$neighborhood  = $params['NMBAIRCONS'];
			$cityName      = $params['NMMUNICIPIO'];
			$stateCode     = strlen($params['SGESTADO']) > 2 ? $this->util->mapState($params['SGESTADO']) : $params['SGESTADO'];
			$addressLat    = null;
      $addressLng    = null;
      $resp = $this->util->getCoordinatesByAddress($streetAddress, $params['NRENDECONS'], $neighborhood, $cityName, $stateCode);

			if ($this->util->verifyLocationService($resp)) {
          $location   = $resp['results'][0]['geometry']['location'];
          $addressLat = $location['lat'];
          $addressLng = $location['lng'];
      }

			$params['NRLATENDE'] = $addressLat;
			$params['NRLONGENDE'] = $addressLng;
			$params['NMBAIRCONS'] = $this->formatNeightbor($neighborhood);
			return $this->addressFactory->addAdditionalAddress($params, $addressCode);
    }

    public function formatNeightbor($neightbor) {
		$formatNeighbor = $neightbor;
		$dictionary = self::dictionary;
		
		if (strlen($neightbor) >= 30){
			$formatNeighbor = $this->util->removeAcentos($formatNeighbor);
			foreach($dictionary as $key => $value){
				$formatNeighbor = str_replace($value, $key, strtoupper($formatNeighbor));
			}
			
			if (strlen($formatNeighbor) >= 30) {
				$formatNeighbor = substr($formatNeighbor, 0, 30);
			}
		}
		
		return $formatNeighbor;
	}

    const dictionary = array (
		"ACAMP"           => "ACAMPAMENTO",
		"AC"              => "ACESSO",
		"AD"              => "ADRO",
		"ERA"             => "AEROPORTO",
		"AL"              => "ALAMEDA",
		"AT"              => "ALTO",
		"A"               => "AREA",
		"AE"              => "AREA ESPECIAL",
		"AV"              => "AVENIDA",
		"AV-CONT"         => "AVENIDA CONTORNO",
		"BX"              => "BAIXA",
		"BLO"             => "BALAO",
		"BAL"             => "BALNEARIO",
		"BC"              => "BECO",
		"BELV"            => "BELVEDERE",
		"BL"              => "BLOCO",
		"BSQ"             => "BOSQUE",
		"BVD"             => "BOULEVARD",
		"BCO"             => "BURACO",
		"C"               => "CAIS",
		"CALC"            => "CALCADA",
		"CAM"             => "CAMINHO",
		"CPO"             => "CAMPO",
		"CAN"             => "CANAL",
		"CHAP"            => "CHACARA",
		"CHAP"            => "CHAPADAO",
		"CIRC"            => "CIRCULAR",
		"COL"             => "COLONIA",
		"CMP-VR"          => "COMPLEXO VIARIO",
		"COND"            => "CONDOMINIO",
		"CJ"              => "CONJUNTO",
		"COR"             => "CORREDOR",
		"CRG"             => "CORREGO",
		"DSC"             => "DESCIDA",
		"DSV"             => "DESVIO",
		"DT"              => "DISTRITO",
		"EVD"             => "ELEVADA",
		"EQ"              => "ENTRE QUADRA",
		"ESC"             => "ESCADA",
		"ESP"             => "ESPLANADA",
		"ETC"             => "ESTACAO",
		"ESTC"            => "ESTACIONAMENTO",
		"ETD"             => "ESTADIO",
		"ETN"             => "ESTANCIA",
		"EST"             => "ESTRADA",
		"EST-MUN ESTRADA" => "MUNICIPAL",
		"FAV"             => "FAVELA",
		"FAZ"             => "FAZENDA",
		"FRA"             => "FEIRA",
		"FER"             => "FERROVIA",
		"FNT"             => "FONTE",
		"FTE"             => "FORTE",
		"GAL"             => "GALERIA",
		"GJA"             => "GRANJA",
		"HAB"             => "HABITACIONAL",
		"IA"              => "ILHA",
		"JD"              => "JARDIM",
		"JD"			  => "JARDINS",
		"JDE"             => "JARDINETE",
		"LD"              => "LADEIRA",
		"LG"              => "LAGO",
		"LGA"             => "LAGOA",
		"LRG"             => "LARGO",
		"LOT"             => "LOTEAMENTO",
		"MNA"             => "MARINA",
		"MOD"             => "MODULO",
		"TEM"             => "MONTE",
		"MRO"             => "MORRO",
		"NUC"             => "NUCLEO",
		"PDA"             => "PARADA",
		"PAR"             => "PARALELA",
		"PRQ"             => "PARQUE",
		"PSG"             => "PASSAGEM",
		"PSA"             => "PASSARELA",
		"PAS"             => "PASSEIO",
		"PAT"             => "PATIO",
		"PNT"             => "PONTA",
		"PTE"             => "PONTE",
		"PTO"             => "PORTO",
		"PC"              => "PRACA",
		"PR"              => "PRAIA",
		"PRL"             => "PROLONGAMENTO",
		"Q"               => "QUADRA",
		"QTA"             => "QUINTA",
		"QTAS"            => "QUINTA",
		"RMP"             => "RAMPA",
		"REC"             => "RECANTO",
		"RES"             => "RESIDENCIAL",
		"RET"             => "RETA",
		"RER"             => "RETIRO",
		"RTN"             => "RETORNO",
		"ROD-AN"          => "RODO ANEL",
		"ROD"             => "RODOVIA",
		"RTT"             => "ROTATORIA",
		"ROT"             => "ROTULA",
		"R"               => "RUA",
		"ST"              => "SANTO",
		"SIT"             => "SITIO",
		"SUB"             => "SUBIDA",
		"TER"             => "TERMINAL",
		"TV"              => "TRAVESSA",
		"TV-PART"         => "TRAVESSA PARTICULAR",
		"TRV"             => "TRECHO",
		"TRV"             => "TREVO",
		"TCH"             => "TRINCHEIRA",
		"TUN"             => "TUNEL",
		"UNID"            => "UNIDADE",
		"VAL"             => "VALA",
		"VLE"             => "VALE",
		"VER"             => "VEREDA",
		"V"               => "VIA",
		"V-AC"            => "VIA DE ACESSO",
		"V-PED"           => "VIA DE PEDESTRE",
		"V-EVD"           => "VIA ELEVADO",
		"V-EXP"           => "VIA EXPRESSA",
		"VD"              => "VIADUTO",
		"VLA"             => "VIELA",
		"VL"              => "VILA"
	);

	public function deleteAdditionalAddress($params) {
		if (empty($params['CDENDECONS'])) {
			throw new \Exception("Código do endereço não foi informado.");
		}

		if (empty($params['CDCLIENTE'])) {
			throw new \Exception("Código do cliente não foi informado.");
		}

		if (empty($params['CDCONSUMIDOR'])) {
			throw new \Exception("Código do consumidor não foi informado.");
		}

		return $this->addressFactory->deleteAdditionalAddress($params);
	}

	public function updateAddress($params) {
		if (empty($params['CDENDECONS'])) {
			throw new \Exception("Código do endereço não foi informado.");
		}

		if (empty($params['CDCLIENTE'])) {
			throw new \Exception("Código do cliente não foi informado.");
		}

		if (empty($params['CDCONSUMIDOR'])) {
			throw new \Exception("Código do consumidor não foi informado.");
		}

		$streetAddress = $params['DSENDECONS'];
		$neighborhood  = $params['NMBAIRCONS'];
		$cityName      = isset($params['NMMUNICIPIO']);
    $stateCode     = isset($params['SGESTADO']);
    $addressLat    = null;
    $addressLng    = null;
    $resp = $this->util->getCoordinatesByAddress($streetAddress, $params['NRENDECONS'], $neighborhood, $cityName, $stateCode);

    if ($this->util->verifyLocationService($resp)) {
        $location   = $resp['results'][0]['geometry']['location'];
        $addressLat = $location['lat'];
        $addressLng = $location['lng'];
      }

    $params['NRLATENDE'] = $addressLat;
		$params['NRLONGENDE'] = $addressLng;
		$params['NMBAIRCONS'] = $this->formatNeightbor($neighborhood);

		if ($params['CDENDECONS'] === 'PRINCIPAL') {
			$this->addressFactory->updateAddress($params);
		} else {
			$this->addressFactory->updateAdditionalAddress($params);
		}
	}
}